const GLib = imports.gi.GLib;
const Gtk = imports.gi.Gtk;
const Gdk = imports.gi.Gdk;

const ExtensionUtils = imports.misc.extensionUtils;

const GTK_VERSION = Gtk.get_major_version();

function init() {
}

function buildPrefsWidget() {
    let widget = new PrefsWidget();
    	if(GTK_VERSION == 3) {
    		widget.widget.show_all(); }
    		
    return widget.widget;
}

class PrefsWidget {
    constructor() {
        this._settings = ExtensionUtils.getSettings();
        this.BC_button = new Gtk.ColorButton();
        this._setButtonColor();

        this.widget = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
        });

        this.vbox = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            hexpand: true,
        });
        
        if(GTK_VERSION == 3) {
        	this.vbox.add(this.selectButtonColor());
        	this.vbox.add(this.customColorButton());
        	this.vbox.add(this.adjustIconSize());
        	this.vbox.add(this.adjustPadding());
        	this.vbox.add(this.adjustBorderRadius());
        	this.vbox.add(this.resetButton());
        	this.vbox.add(new Gtk.Separator({ orientation: Gtk.Orientation.HORIZONTAL, margin_bottom: 10, margin_top: 10 }));
        	this.vbox.add(this.forceLockDown());
        	this.vbox.add(this.removeSuspendButton());
        	this.vbox.add(this.removeLogoutButton());
        	this.vbox.add(this.removeRestartButton());
        	this.vbox.add(this.removePoweroffButton());
        	this.vbox.add(this.lockdownTip());
        	this.vbox.add(new Gtk.Separator({ orientation: Gtk.Orientation.HORIZONTAL, margin_bottom: 10, margin_top: 10 }));
        	this.vbox.add(this.tip());
        	this.vbox.add(this.command());
        
        	
        	this.widget.add(this.vbox);        
        }
        
        else if(GTK_VERSION == 4) {
        	this.vbox.append(this.selectButtonColor());
        	this.vbox.append(this.customColorButton());
        	this.vbox.append(this.adjustIconSize());
        	this.vbox.append(this.adjustPadding());
        	this.vbox.append(this.adjustBorderRadius());
        	this.vbox.append(this.resetButton());
        	this.vbox.append(new Gtk.Separator({ orientation: Gtk.Orientation.HORIZONTAL, margin_bottom: 10, margin_top: 10 }));
        	this.vbox.append(this.forceLockDown());
        	this.vbox.append(this.removeSuspendButton());
        	this.vbox.append(this.removeLogoutButton());
        	this.vbox.append(this.removeRestartButton());
        	this.vbox.append(this.removePoweroffButton());
        	this.vbox.append(this.lockdownTip());
        	this.vbox.append(new Gtk.Separator({ orientation: Gtk.Orientation.HORIZONTAL, margin_bottom: 10, margin_top: 10 }));
        	this.vbox.append(this.tip());
        	this.vbox.append(this.command());
        	
        	this.widget.append(this.vbox);
    	}
    }
    
_setButtonColor() {
        let rgba = new Gdk.RGBA();
        let hexString = this._settings.get_string('color');
        rgba.parse(hexString);
        this.BC_button.set_rgba(rgba);
    }
    
    
_cssHexString(css) {
        let rrggbb = '#';
        let start;
        for(let loop = 0; loop < 3; loop++) {
            let end = 0;
            let xx = '';
            for(let loop = 0; loop < 2; loop++) {
                while(true) {
                    let x = css.slice(end, end + 1);
                    if(x == '(' || x == ',' || x == ')')
                        break;
                    end = end + 1;
                }
                if(loop == 0) {
                    end = end + 1;
                    start = end;
                }
            }
            xx = parseInt(css.slice(start, end)).toString(16);
            if(xx.length == 1)
                xx = '0' + xx;
            rrggbb = rrggbb + xx;
            css = css.slice(end);
        }
        return rrggbb;
    }
    
_onPanelColorChanged() {
        let rgba = this.BC_button.get_rgba();
        let css = rgba.to_string();
        let hexString = this._cssHexString(css);
        this._settings.set_string('color', hexString);
    }    
    
selectButtonColor() {
	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.END  });
	
	this.BC_button.connect('notify::rgba', ()=> this._onPanelColorChanged() );
	
	if(GTK_VERSION == 3) {
		hbox.add(this.BC_button);
		}
		
	else if(GTK_VERSION == 4) {
		hbox.append(this.BC_button);
		}

        return hbox;
    }
    
customColorButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
    	let customColorButtonLabel = new Gtk.Label({ label: "Use Custom Color for Buttons", xalign: 0, hexpand: true });
    	
        this.cCBToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('use-custom-color') });
        this.cCBToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('use-custom-color', button.active); });
    	
    	if(GTK_VERSION == 3) {
		hbox.add(customColorButtonLabel);
		hbox.add(this.cCBToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(customColorButtonLabel);
		hbox.append(this.cCBToggleSwitch) }
      	
      	return hbox;
    } 
    
forceLockDown() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.CENTER   });
    	let forceLockDownLabel = new Gtk.Label({ label: "<Force Lockdown Keys> - Turn ON to Control Below 4 Buttons      ", xalign: 0, hexpand: true });
    	
        this.fLDToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('force-lock-down') });
        this.fLDToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('force-lock-down', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(forceLockDownLabel);
		hbox.add(this.fLDToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(forceLockDownLabel);
		hbox.append(this.fLDToggleSwitch) }
      	
      	return hbox;
    } 
    
removeSuspendButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5  });
    	let removeSuspendButtonLabel = new Gtk.Label({ label: "Remove Suspend Button", xalign: 0, hexpand: true });
    	
        this.rSBLToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('remove-suspend-button') });
        this.rSBLToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('remove-suspend-button', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(removeSuspendButtonLabel);
		hbox.add(this.rSBLToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(removeSuspendButtonLabel);
		hbox.append(this.rSBLToggleSwitch) }
      	
      	return hbox;
    } 
    
removeLogoutButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
    	let removeLogoutButtonLabel = new Gtk.Label({ label: "Remove Logout Button", xalign: 0, hexpand: true });
    	
        this.rLBLToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('remove-logout-button') });
        this.rLBLToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('remove-logout-button', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(removeLogoutButtonLabel);
		hbox.add(this.rLBLToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(removeLogoutButtonLabel);
		hbox.append(this.rLBLToggleSwitch) }
      	
      	return hbox;
    }
    
removeRestartButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
    	let removeRestartButtonLabel = new Gtk.Label({ label: "Remove Restart Button", xalign: 0, hexpand: true });
    	
        this.rRBLToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('remove-restart-button') });
        this.rRBLToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('remove-restart-button', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(removeRestartButtonLabel);
		hbox.add(this.rRBLToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(removeRestartButtonLabel);
		hbox.append(this.rRBLToggleSwitch) }
      	
      	return hbox;
    } 
    
removePoweroffButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
    	let removePoweroffButtonLabel = new Gtk.Label({ label: "Remove Poweroff Button", xalign: 0, hexpand: true });
    	
        this.rPBLToggleSwitch = new Gtk.Switch({ active: this._settings.get_boolean('remove-poweroff-button') });
        this.rPBLToggleSwitch.connect('notify::active', (button) => { this._settings.set_boolean('remove-poweroff-button', button.active); }); 
    	
    	if(GTK_VERSION == 3) {
		hbox.add(removePoweroffButtonLabel);
		hbox.add(this.rPBLToggleSwitch) }
		
        else if(GTK_VERSION == 4) {
		hbox.append(removePoweroffButtonLabel);
		hbox.append(this.rPBLToggleSwitch) }
      	
      	return hbox;
    } 
    
lockdownTip() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.START  });
    	
    	let lockdownTipLabel = new Gtk.Label({
    		xalign: 0,
    		useMarkup: true,
    		wrap: true,
    		label: '<b>This below note is applicable only if "Force Lockdown Keys" switch above is Turned ON</b>\n\
if your systems gsettings schema org.gnome.desktop.lockdown with key disable-log-out is set to true(default is false) then\n\
LOGOUT, RESTART, POWEROFF will not take any Action when you click on them. Also in Special Case the\n\
SUSPEND button will not take any Action.'
    		});
    	
    	if(GTK_VERSION == 3) {
		hbox.add(lockdownTipLabel); }
	else if(GTK_VERSION == 4) {
		hbox.append(lockdownTipLabel); }
		
        return hbox;
    } 
    
adjustIconSize() {
	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
	let IS_label = new Gtk.Label({ label: "Adjust Icon Size", xalign: 0, hexpand: true });

	this.IS_button = new Gtk.SpinButton();
	this.IS_button.set_range(12, 20);
	this.IS_button.set_value(this._settings.get_int('icon-size'));
	this.IS_button.set_increments(2, 4);
	this.IS_button.connect('value-changed', (entry) => { this._settings.set_int('icon-size', entry.get_value()); });
	
	if(GTK_VERSION == 3) {
		hbox.add(IS_label);
		hbox.add(this.IS_button); }
		
	else if(GTK_VERSION == 4) {
		hbox.append(IS_label);
		hbox.append(this.IS_button); }

        return hbox;
    }
    
adjustPadding() {
        let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
        let PD_label = new Gtk.Label({ label: "Adjust Padding", xalign: 0, hexpand: true });

        this.PD_button = new Gtk.SpinButton();
	this.PD_button.set_range(0, 16);
	this.PD_button.set_value(this._settings.get_int('padding'));
	this.PD_button.set_increments(2, 4);
	this.PD_button.connect('value-changed', (entry) => { this._settings.set_int('padding', entry.get_value()); });
	
	if(GTK_VERSION == 3) {
		hbox.add(PD_label);
		hbox.add(this.PD_button); }
		
	else if(GTK_VERSION == 4) {
		hbox.append(PD_label);
		hbox.append(this.PD_button); }

        return hbox;
    }  
    
adjustBorderRadius() {
	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
	let BR_label = new Gtk.Label({ label: "Adjust Border Raidus", xalign: 0, hexpand: true });
	this.BR_button = new Gtk.SpinButton();
	this.BR_button.set_range(0, 32);
	this.BR_button.set_value(this._settings.get_int('border-radius'));
	this.BR_button.set_increments(2, 4);
	this.BR_button.connect('value-changed', (entry) => { this._settings.set_int('border-radius', entry.get_value()); });
	
	if(GTK_VERSION == 3) {
		hbox.add(BR_label);
		hbox.add(this.BR_button); }
		
	else if(GTK_VERSION == 4) {
		hbox.append(BR_label);
		hbox.append(this.BR_button); }

        return hbox;
    }
    
resetButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.CENTER  });
    	let resetButton = new Gtk.Button();
        resetButton.set_label("Reset to Extensions's Default Size'es By Cliking this Button");
        resetButton.connect('clicked', ()=> {	
        	this._settings.set_int('icon-size', 16);
        	this._settings.set_int('padding', 8);
        	this._settings.set_int('border-radius', 16);
			this.IS_button.set_value(this._settings.get_int('icon-size'));
			this.PD_button.set_value(this._settings.get_int('padding'));
			this.BR_button.set_value(this._settings.get_int('border-radius'));
			
       		});
	
	if(GTK_VERSION == 3) {
		hbox.add(resetButton); }
	else if(GTK_VERSION == 4) {
		hbox.append(resetButton) }

        return hbox;
    }  
    
tip() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.START  });
    	let tipLabel = new Gtk.Label({ label:
    	"you can tweak the BACKGROUND COLOR of these system menu buttons by edititng this extensions stylesheet.css file.\n\
Run below command to open the styles sheet file." });
    	
    	if(GTK_VERSION == 3) {
		hbox.add(tipLabel); }
	else if(GTK_VERSION == 4) {
		hbox.append(tipLabel); }
		
        return hbox;
    }     
    
command() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, margin_bottom: 20, halign: Gtk.Align.START  });
    	let command = new Gtk.Label({ label: "<b>gedit $HOME/.local/share/gnome-shell/extensions/roundedSystemMenuButtons@pratap.fastmail.fm/stylesheet.css</b>", useMarkup: true, selectable: true });
    	
    	if(GTK_VERSION == 3) {
		hbox.add(command); }
        else if(GTK_VERSION == 4) {
        	hbox.append(command); }
      	
      	return hbox;
    } 
    
   
}

